<?php

include_once 'j_config.php';

function sec_session_start() {
    $session_name = 'sec_session_id';   // OTORGA UN NOMBRE SE SESION
    $secure = SECURE;

    // FUERZA A COMANDOS JAVASCRIPT A INICIAR SESSION MEDIANTE ID//
    $httponly = true;

    // FUERZA  A LAS SESIONES A USAR SOLO COOKIES.
    if (ini_set('session.use_only_cookies', 1) === FALSE) {
        header("Location: ../error.php?err=Could not initiate a safe session (ini_set)");
        exit();
    }

    // CONFIGURE PARAMETROS DE COOKIES//
    $cookieParams = session_get_cookie_params();
    session_set_cookie_params($cookieParams["lifetime"], $cookieParams["path"], $cookieParams["domain"], $secure, $httponly);

    // ESTABLECE UN NOMBRE DE SESION GLOBAL.
    session_name($session_name);

    session_start();            // INICIA SESION EN PHP 
    session_regenerate_id();    // REGENERA LA SESION Y BORRA LA ANTERIOR. 
}

function login($email, $password, $mysqli) {
    // USO DE SENTENCIAS PREPARADAS PARA EVITAR INYECCION SQL 
    if ($stmt = $mysqli->prepare("SELECT id, username, password, salt 
				  FROM sisfoto_usuarios 
                                  WHERE email = ? LIMIT 1")) {
        $stmt->bind_param('s', $email);  // UNE "$email" A LOS PARAMETROS.
        $stmt->execute();    // EJECUTA LA SENTENCIA PREPARADA.
        $stmt->store_result();

        // OBTIENE LAS VARIABLES DEL RESULTADO.
        $stmt->bind_result($user_id, $username, $db_password, $salt);
        $stmt->fetch();

        // LA CONTRASEÑA HASH CON UN UNICO SALT.
        $password = hash('sha512', $password . $salt);
        if ($stmt->num_rows == 1) {
            // SI EL USUARIO EXISTE, VERIFICAMOS SI LA CUENTA ESTÁ BLOQUEADA

            // DE DEMASIADOS INTENTOS DE INICIO DE SESIÓN 
            if (checkbrute($user_id, $mysqli) == true) {
                // LA CUENTA ESTA BLOQUEADA 
                // SE ENVIARA UN CORREO ELECTRÓNICO AL USUARIO DICIENDO QUE SU CUENTA ESTÁ BLOQUEADA 
                return false;
            } else {
                // COMPRUEBE SI LA CONTRASEÑA DE LA BASE DE DATOS COINCIDE
                // LA CONTRASEÑA QUE EL USUARIO HA ENVIADO.
                if ($db_password == $password) {
                    // LA CONTRASEÑA ES CORRECTA
                    // OBTENER LA CADENA USER-AGENT DEL USUARIO.
                    $user_browser = $_SERVER['HTTP_USER_AGENT'];

                    // PROTECCIÓN XSS, YA QUE PODRÍAMOS IMPRIMIR ESTE VALOR
                    $user_id = preg_replace("/[^0-9]+/", "", $user_id);
                    $_SESSION['user_id'] = $user_id;

                    // PROTECCIÓN XSS, YA QUE PODRÍAMOS IMPRIMIR ESTE VALOR
                    $username = preg_replace("/[^a-zA-Z0-9_\-]+/", "", $username);

                    $_SESSION['username'] = $username;
                    $_SESSION['login_string'] = hash('sha512', $password . $user_browser);

                    // INICIO DE SESION CORRECTO
                    return true;
                } else {
                    // LA CONTRASEÑA NO ES CORRECTA
                    // REGISTRAMOS ESTE INTENTO EN LA BASE DE DATOS 
                    $now = time();
                    if (!$mysqli->query("INSERT INTO sisfoto_loger(user_id, time) 
                                    VALUES ('$user_id', '$now')")) {
                        header("Location: ../error.php?err=Database error: sisfoto_loger");
                        exit();
                    }

                    return false;
                }
            }
        } else {
            // NO EXISTE EL USUARIO. 
            return false;
        }
    } else {
        // NO SE PUDO CREAR UNA DECLARACIÓN PREPARADA
        header("Location: ../error.php?err=Database error: cannot prepare statement");
        exit();
    }
}

function checkbrute($user_id, $mysqli) {
    // OBTENER LA MARCA DE TIEMPO DE LA HORA ACTUAL
    $now = time();

    // TODOS LOS INTENTOS DE INICIO DE SESIÓN SE CUENTAN DESDE LAS ÚLTIMAS 2 HORAS.. 
    $valid_attempts = $now - (2 * 60 * 60);

    if ($stmt = $mysqli->prepare("SELECT time 
                                  FROM sisfoto_loger 
                                  WHERE user_id = ? AND time > '$valid_attempts'")) {
        $stmt->bind_param('i', $user_id);

        // EJECUTA LA SENTENCIA PREPARADA 
        $stmt->execute();
        $stmt->store_result();

        // SI HA HABIDO MÁS DE 5 INICIOS DE SESIÓN FALLIDOS 
        if ($stmt->num_rows > 5) {
            return true;
        } else {
            return false;
        }
    } else {
        // SI NO SE PUEDE CREAR UNA DECLARACIÓN PREPARADA
        header("Location: ../error.php?err=Database error: cannot prepare statement");
        exit();
    }
}

function login_check($mysqli) {
    // COMPRUEBE SI TODAS LAS VARIABLES DE SESIÓN ESTÁN CONFIGURADAS 
    if (isset($_SESSION['user_id'], $_SESSION['username'], $_SESSION['login_string'])) {
        $user_id = $_SESSION['user_id'];
        $login_string = $_SESSION['login_string'];
        $username = $_SESSION['username'];

        // OBTENER LA CADENA USER-AGENT DEL USUARIO.
        $user_browser = $_SERVER['HTTP_USER_AGENT'];

        if ($stmt = $mysqli->prepare("SELECT password 
				      FROM sisfoto_usuarios 
				      WHERE id = ? LIMIT 1")) {
            // UNE "$user_id" A LOS PARAMETROS. 
            $stmt->bind_param('i', $user_id);
            $stmt->execute();   // EJECUTA LA SENTENCIA PREPARADA.
            $stmt->store_result();

            if ($stmt->num_rows == 1) {
                // SI EL USUARIO EXISTE GET VARIABLES FROM RESULT.
                $stmt->bind_result($password);
                $stmt->fetch();
                $login_check = hash('sha512', $password . $user_browser);

                if ($login_check == $login_string) {
                    // CONECTADO!!!! 
                    return true;
                } else {
                    // NO SE PUDO CONECTAR 
                    return false;
                }
            } else {
                // NO SE PUDO CONECTAR
                return false;
            }
        } else {
            // NO SE PUDO PREPARAR LA DECLARACIÓN
            header("Location: ../error.php?err=Database error: cannot prepare statement");
            exit();
        }
    } else {
        // NO SE PUDO CONECTAR
        return false;
    }
}

function esc_url($url) {

    if ('' == $url) {
        return $url;
    }

    $url = preg_replace('|[^a-z0-9-~+_.?#=!&;,/:%@$\|*\'()\\x80-\\xff]|i', '', $url);
    
    $strip = array('%0d', '%0a', '%0D', '%0A');
    $url = (string) $url;
    
    $count = 1;
    while ($count) {
        $url = str_replace($strip, '', $url, $count);
    }
    
    $url = str_replace(';//', '://', $url);

    $url = htmlentities($url);
    
    $url = str_replace('&amp;', '&#038;', $url);
    $url = str_replace("'", '&#039;', $url);

    if ($url[0] !== '/') {
        // SOLO ESTAMOS INTERESADOS EN ENLACES RELATIVOS DESDE $_SERVER['PHP_SELF']
        return '';
    } else {
        return $url;
    }
}
